#include "gamebase.hpp"
#include "texture.hpp"
#include "sdlmain.hpp"
#include <sstream>
#include <SDL2/SDL.h>
#include <ciso646>

namespace cloonel {
	namespace {
		///---------------------------------------------------------------------
		///---------------------------------------------------------------------
		bool DoEvents() {
			SDL_Event eve;
			while (SDL_PollEvent(&eve)) {
				switch (eve.type) {
				case SDL_QUIT:
					return true;
				}
			}
			return false;
		}
	} //unnamed namespace

	///------------------------------------------------------------------------
	///------------------------------------------------------------------------
	GameBase::GameBase (SDLMain* parSdlMain) :
		m_sdlmain(parSdlMain),
		m_time0(SDL_GetTicks()),
		m_wantsToQuit(false)
	{
	}

	///------------------------------------------------------------------------
	///------------------------------------------------------------------------
	GameBase::~GameBase() {
	}

	///------------------------------------------------------------------------
	///------------------------------------------------------------------------
	std::unique_ptr<Texture> GameBase::LoadTexture (const char* parPath) {
		std::ostringstream oss;
		oss << "/home/duckz/dev/code/cpp/clooneljump/" << parPath;
		return std::move(std::unique_ptr<Texture>(new Texture(oss.str(), m_sdlmain, true)));
	}

	///------------------------------------------------------------------------
	///------------------------------------------------------------------------
	float GameBase::Exec() {
		const unsigned int time1 = SDL_GetTicks();
		const float delta = static_cast<float>(time1 - m_time0) * 0.001;
		m_time0 = time1;

		SDL_Renderer* const ren = m_sdlmain->GetRenderer();

		SDL_RenderClear(ren);
		OnUpdate(delta);
		OnRender();
		SDL_RenderPresent(ren);

		m_wantsToQuit = DoEvents();

		return delta;
	}

	///------------------------------------------------------------------------
	///------------------------------------------------------------------------
	bool GameBase::WantsToQuit() const {
		return m_wantsToQuit or ShouldQuit();
	}

	///------------------------------------------------------------------------
	///------------------------------------------------------------------------
	bool GameBase::ShouldQuit() const {
		return false;
	}
} //namespace cloonel
